/*
 * @brineAvgStruct.java Version 1.1 05/16/2014
 *
 * Copyright (c) 2014 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine;

import brine.brineListStruct;

/** CLASS brineAvgStruct
 *  <p> This Class holds the Averages of Plotted Brine Data Information.
 *
 *  @version 1.1 05/16/2014
 *  @author  John R. Victorine
 */

public class brineAvgStruct
{
  public static final int _K      = brine.brineStandardTools._K;     // Potassium
  public static final int _NA     = brine.brineStandardTools._NA;    // Sodium
  public static final int _NA_K   = brine.brineStandardTools._NA_K;  // Sodium/Potasium
  public static final int _MG     = brine.brineStandardTools._MG;    // Magnesium
  public static final int _CA     = brine.brineStandardTools._CA;    // Calcium
  public static final int _FE_II  = brine.brineStandardTools._FE_II; // Iron (II)

  public static final int _HCO3   = brine.brineStandardTools._HCO3;  // Bicarbonate
  public static final int _CO3    = brine.brineStandardTools._CO3;   // Carbonate
  public static final int _SO4    = brine.brineStandardTools._SO4;   // Sulfate
  public static final int _CL     = brine.brineStandardTools._CL;    // Chloride
  public static final int _BR     = brine.brineStandardTools._BR;    // Bromide
  public static final int _I      = brine.brineStandardTools._I;     // Iodide

  public static final int _OHM    = brine.brineStandardTools._OHM;   // Resistivity (Rw)
  public static final int _PH     = brine.brineStandardTools._PH;    // PH

  public static final int _ORD[]  = {
	  _K, _NA, _NA_K, _MG, _CA, _FE_II, _HCO3, _CO3, _SO4, _CL, _BR, _I, _OHM, _PH
  };

  public static final int ROWS    = 14;
  public static final int COLUMNS = 8;

  public static final int _MINIMUM = 0;
  public static final int _5       = 1;
  public static final int _25      = 2;
  public static final int _MEDIAN  = 3;
  public static final int _MEAN    = 4;
  public static final int _75      = 5;
  public static final int _95      = 6;
  public static final int _MAXIMUM = 7;

  /** Method getData()
   *  <p> This method will create the Averages Data
   *
   *  Columns -> Min, 5%, 25%,  Median, Mean, 75%, 95%, Max
   *
   *  @param  st   = Brine Data List Structure
   *  @return data = array of averages for primary data
   */

  public static double[][] getData( brineListStruct st )
  {
	int i,j,k;
	double data[][] = null;
	double dTemp[]  = null;
	double dTemp1[] = null;
	double dSum     = 0.0;
	int    iTotal   = 0;
	int    iCount   = 0;

    if (st != null)
    {
  	  data = new double[ROWS][COLUMNS];

	  for (i=0; i<ROWS; i++)
	  {
	    for (j=0; j<COLUMNS; j++)
	    {
	  	  data[i][j] = 0.0;
	    }
	  }

      if (st.iCount > 0)
      {
        for (i=0; i<ROWS; i++)
        {
    	  dTemp = new double[st.iCount];

		  for (k=0; k<st.iCount; k++)
		    dTemp[k] = 0.0;

		  if (st.checkData(_ORD[i]))
		  {
		    dTemp = st.getData(_ORD[i]);
		    dTemp = bubbleSort(dTemp);

            iTotal = 0;
		    for (k=0; k<st.iCount; k++)
		    {
			  if (dTemp[k] > 0.0)
			    iTotal++;
			}

            dTemp1 = new double[iTotal];

            iCount = 0;
            for (k=0; k<st.iCount; k++)
            {
			  if (dTemp[k] > 0.0)
			  {
  			    dTemp1[iCount] = dTemp[k];
  			    iCount++;
			  }
			}

            if (iCount > 0)
            {
              dTemp = new double[iCount];

		      for (k=0; k<iCount; k++)
		      {
			    dTemp[k] = dTemp1[k];
			  }

		      data[i][_MINIMUM] = dTemp[0];
		      data[i][_MAXIMUM] = dTemp[iCount-1];

              dSum = 0.0;
  		      for (k=0; k<iCount; k++)
		        dSum = dSum + dTemp[k];
		      data[i][_MEAN] = dSum / iCount;

		      data[i][_5]      = computePercentile(iCount, 0.05, dTemp);
		      data[i][_25]     = computePercentile(iCount, 0.25, dTemp);
		      data[i][_MEDIAN] = computePercentile(iCount, 0.5,  dTemp);
		      data[i][_75]     = computePercentile(iCount, 0.75, dTemp);
		      data[i][_95]     = computePercentile(iCount, 0.95, dTemp);
			}
		  }
	    }
	  }
	}

	return (data);
  }

  /** Method getData()
   *  <p> This method will create the Averages Data
   *
   *  Columns -> Min, 5%, 25%,  Median, Mean, 75%, 95%, Max
   *
   *  @param  st     = Brine Data List Structure
   *  @param  iColor = Array of Brine Data Identifiers
   *  @return data   = array of averages for primary data
   */

  public static double[][] getData( brineListStruct st, int iColor[] )
  {
	int i,j,k;
	double data[][] = null;
	double dTemp[]  = null;
	double dTemp1[] = null;
	double dSum     = 0.0;
	int    iTotal   = 0;
	int    iCount   = 0;
    int    iRows    = 0;

    if (st != null)
    {
	  iRows = iColor.length;
  	  data  = new double[iRows][COLUMNS];

	  for (i=0; i<iRows; i++)
	  {
	    for (j=0; j<COLUMNS; j++)
	    {
	  	  data[i][j] = 0.0;
	    }
	  }

      if (st.iCount > 0)
      {
        for (i=0; i<iRows; i++)
        {
    	  dTemp = new double[st.iCount];

		  for (k=0; k<st.iCount; k++)
		    dTemp[k] = 0.0;

		  if (st.checkData(iColor[i]))
		  {
		    dTemp = st.getData(iColor[i]);
		    dTemp = bubbleSort(dTemp);

            iTotal = 0;
		    for (k=0; k<st.iCount; k++)
		    {
			  if (dTemp[k] > 0.0)
			    iTotal++;
			}

            dTemp1 = new double[iTotal];

            iCount = 0;
            for (k=0; k<st.iCount; k++)
            {
			  if (dTemp[k] > 0.0)
			  {
  			    dTemp1[iCount] = dTemp[k];
  			    iCount++;
			  }
			}

            if (iCount > 0)
            {
              dTemp = new double[iCount];

		      for (k=0; k<iCount; k++)
		      {
			    dTemp[k] = dTemp1[k];
			  }

		      data[i][_MINIMUM] = dTemp[0];
		      data[i][_MAXIMUM] = dTemp[iCount-1];

              dSum = 0.0;
  		      for (k=0; k<iCount; k++)
		        dSum = dSum + dTemp[k];
		      data[i][_MEAN] = dSum / iCount;

		      data[i][_5]      = computePercentile(iCount, 0.05, dTemp);
		      data[i][_25]     = computePercentile(iCount, 0.25, dTemp);
		      data[i][_MEDIAN] = computePercentile(iCount, 0.5,  dTemp);
		      data[i][_75]     = computePercentile(iCount, 0.75, dTemp);
		      data[i][_95]     = computePercentile(iCount, 0.95, dTemp);
			}
		  }
	    }
	  }
	}

	return (data);
  }

  /** Method computePercentile()
   *  <p> This method will compute the Percentile value
   *  @param iTotal   = the Total number of rows
   *  @param dPercent = the decimal percentage value
   *  @param data     = array of numbers presorted
   */

  public static double computePercentile(
	                     int iTotal, double dPercent, double data[])
  {
	double dValue    = 0.0;
	double RowNum    = 0.0;
	double CelRowNum = 0.0;
	double FlrRowNum = 0.0;
	int    iFlr      = 0;
	int    iCel      = 0;

	if ((iTotal > 0) && data != null)
	{
	  RowNum    = 1.0 + (dPercent * (iTotal-1));
	  CelRowNum = Math.ceil(RowNum);
	  FlrRowNum = Math.floor(RowNum);

	  if ((CelRowNum == RowNum) && (FlrRowNum == RowNum))
	  {
		if (RowNum < data.length)
	      dValue = data[(int) RowNum];
	  }
	  else
	  {
		iFlr = (int) FlrRowNum;
		if (iFlr >= iTotal)
		  iFlr = iTotal-1;

		iCel = (int) CelRowNum;
		if (iCel >= iTotal)
		  iCel = iTotal-1;

		if ((iFlr < data.length) && (iCel < data.length))
		{
  	      dValue = (CelRowNum-RowNum) * data[iFlr] +
	               (RowNum-FlrRowNum) * data[iCel];
		}
	  }
	}

	return (dValue);
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  data = data array to be sorted
   * @return data = sorted data array
   */

  public static double[] bubbleSort(double data[])
  {
    boolean swappedOnPrevRun = true;
    double  dTemp   = 0.0;

    if (data != null)
    {
      if (data.length > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<data.length-1; i++)
          {
            // if current element is greater than the next swap the two elements

            if(data[i] > data[i+1])
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              dTemp = data[i];

              // set element i+1 to where i used to be

              data[i] = data[i+1];

              // release the old i from temp into i+1 slot

              data[i+1] = dTemp;
            }
          }
        }
      }
    }

    return (data);
  }
}