/*
 * @fluidStandardTools.java Version 1.1 03/21/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package fluid;

/** Class fluidStandardTools
 *  <p> This Class holds the standard tool information to map the fluid
 *      curves to standard names for plotting.
 */

public class fluidStandardTools
{
  public static final int _KEY     = 0;  // KEY

  // Other Measured Well Data
  public static final int _FLUID   = 1; // Total Fluid
  public static final int _CUT     = 2; // Oil Cut (%)
  public static final int _BOPD    = 3; // Barrels of Oil per Day
  public static final int _LEVEL   = 4; // Fluid Level (ft)
  public static final int _JNTS    = 5; // Joints in Well
  public static final int _BHP     = 6; // Bottom Hole Pressure
  public static final int _CO2     = 7; // CO2 (%)
  public static final int _MCFD    = 8; // MCF / D

  // Well Header Information Data

  public static final int _NAME   =  9; // Well Name
  public static final int _API    = 10; // API-Number
  public static final int _FIELD  = 11; // Field
  public static final int _COUNTY = 12; // County
  public static final int _STATE  = 13; // State
  public static final int _LOC    = 14; // Location
  public static final int _LAT    = 15; // Latitude
  public static final int _LONG   = 16; // Longitude
  public static final int _KB     = 17; // Kelly Bushing
  public static final int _GL     = 18; // Ground Level
  public static final int _DF     = 19; // Derrick Floor
  public static final int _TGT    = 20; // TGT
  public static final int _SRC    = 21; // Source
  public static final int _DATE   = 22; // Date

  public static final int TOTAL   = 9;  // TOTAL
  public static final int TOTAL_A = 23; // TOTAL With Well Header Data

  // Column Identifiers

  public static final int _C_KEY  = 0;
  public static final int _C_MNEM = 1;
  public static final int _C_DESC = 2;
  public static final int _C_UNIT = 3;
  public static final int _C_TYPE = 4;

  public static final String CURVES[][] =
  {
    // KEY          Mnemonic     Description               Units    TYPE
    { "F_KEY",      "KEY",       "Fluid Primary Key",      "NONE",  "KEY"   },
    { "FLUID",      "FLUID",     "Total Fluid",            "NONE",  "LEVEL" },
    { "F_CUT",      "CUT",       "Oil Cut",                "%",     "LEVEL" },
    { "F_BOPD",     "BOPD",      "Barrels of Oil Per Day", "bbl",   "LEVEL" },
    { "F_LEVEL",    "LEVEL",     "Fluid Level",            "ft",    "LEVEL" },
    { "F_JNTS",     "JNTS",      "Joints in Well",         "NONE",  "LEVEL" },
    { "F_BHP",      "BHP",       "Bottom Hole Pressure",   "psig",  "LEVEL" },
    { "F_CO2",      "CO2",       "CO2",                    "%",     "LEVEL" },
    { "F_MCFD",     "MCFD",      "GAS MCFD",               "mcf/d", "LEVEL" },

    // -- Well ID
    { "B_NAME",     "NAME",      "Well Name",             "NONE", "LOC" },
    { "B_API",      "API",       "API-Number",            "NONE", "LOC" },
    // -- Other Info
    {  "B_FIELD",   "FIELD",     "Field",                 "NONE", "LOC" },
    // -- Location
    {  "B_COUNTY",  "COUNTY",    "County",                "NONE", "LOC" },
    {  "B_STATE",   "STATE",     "State",                 "NONE", "LOC" },
    {  "B_LOC",     "LOC",       "Location",              "NONE", "LOC" },
    {  "B_LAT",     "LAT",       "Latitude",              "deg",  "LOC" },
    {  "B_LONG",    "LONG",      "Longitude",             "deg",  "LOC" },
    // -- Depth Information
    { "B_KB",       "KB",        "Kelly Bushing",         "ft",   "ELEV"  },
    { "B_GL",       "GL",        "Ground Level",          "ft",   "ELEV"  },
    { "B_DF",       "DF",        "Derrick Floor",         "ft",   "ELEV"  },
    { "B_TGT",      "TGT",       "TGT",                   "ft",   "ELEV"  },
    // -- Source Information
    { "B_SRC",      "SRC",       "source",                "NONE", "SAMPLE"  },
    { "B_DATE",     "DATE",      "Recovery date",         "NONE", "SAMPLE"  },
  };
}

/*
 *  @version 1.1 03/21/2016
 *  @author  John Victorine
 */
