/*
 * @fluidStruct.java Version 1.1 03/21/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package fluid;

import fluid.fluidStruct;

/** CLASS fluidStruct
 *  <p> This Class holds the Brine Data Information.
 *
 *  List composed from,
 *  Common Cations, Anions, Acids, Salts and Hydrate Nomenclature
 *  Missouri University of Science and Technology
 *  http://web.mst.edu/~tbone/subjects/tbone/chem1/Chem%201%20Compound%20Sheet%20FS08.pdf
 *
 *  @version 1.1 03/21/2016
 *  @author  John R. Victorine
 */

public class fluidStruct
{
  public static final double dNULL = -999.95; // Null Value for an empty data set

  public String sKID       = "";  // KID
  public String sKEY       = "";  // KEY

  // Location
  public String sWellKID   = "";  // Well Header KID
  public String sAPI       = "";  // API-Number OR UWI
  public String sName      = "";  // Well Name or Outcrop Name

  public double dLatitude  = 0.0; // Latitude of Location
  public double dLongitude = 0.0; // Longitude of Location

  public double dZone      = 0.0; // UTM Zone
  public double dUTMx      = 0.0; // UTM X-Position
  public double dUTMy      = 0.0; // UTM Y-Position

  public String sField     = "";  // Field
  public String sCounty    = "";  // County
  public String state      = "";  // State
  public String sLocation  = "";  // Location

  public double dKB        = 0.0; // Kelly Bushing
  public double dGL        = 0.0; // Ground Level
  public double dDF        = 0.0; // Derrick Floor
  public double dTGT       = 0.0; // TGT

  public String source     = "";  // Sample Source {S}
  public String sRecovery  = "";  // Recovery Date (Date Core Cut) {D}

  // Group
  public String group_id   = "";  // Group ID - LAB or FIELD
  public int    group_no   = -1;  // Group Number - Data organized by Date

  // Other Measured Well Data
  public double dTotal  = dNULL; // Total Fluid
  public double dOilCut = dNULL; // Oil Cut (%)
  public double dBOPD   = dNULL; // Barrels of Oil per Day
  public double dLevel  = dNULL; // Fluid Level (ft)
  public double dJoints = dNULL; // Joints in Well
  public double dBHP    = dNULL; // Estimated Bottom Hole Pressure
  public double dCO2    = dNULL; // CO2 (%)
  public double dMCFD   = dNULL; // MCF/D

  /** Method delete()
   * <p> This method will force set the Objects to null and force Java to
   *     free unwanted memory
   */

  public void delete()
  {
    sKID       = null;  // Primary KEY of Record
    sKEY       = null;  // User defined primary key

    // Location
    sWellKID   = null;  // Well Header KID
    sAPI       = null;  // API-Number OR UWI
    sName      = null;  // Well Name or Outcrop Name
    sField     = null;  // Field
    sCounty    = null;  // County
    state      = null;  // State
    sLocation  = null;  // Location

    source     = null;  // Sample Source {S}
    sRecovery  = null;  // Recovery Date (Date Core Cut) {D}

    // Group

    group_id   = null;  // Group ID - LAB or FIELD
  }
}

/*
 *  @version 1.1 03/21/2016
 *  @author  John Victorine
 */
