/*
 * @gridIORead.java Version 1.1 03/21/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.io;

import java.io.*;
import java.awt.*;
import javax.swing.*;

import iqstrat.iqstratHeadersListStruct;   // Well Header List Structure
import iqstrat.io.ReadHeadersXMLFile;      // Read Well Header XML File

import fluid.fluidListStruct;              // Fluid Data List Structure
import fluid.io.ReadFluidLevelDataXMLFile; // Read Fluid Level Data XML File

/** Class gridIORead
 *  <p> This Class will retrieve the Brine Data from the KGS Database.
 *
 *  @version 1.1 03/21/2016
 *  @author  John R. Victorine
 */

public class gridIORead
{
  /* ===================================================================== *
   * ----------------- LOAD WELL HEADER INFORMATION DATA ----------------- *
   * ===================================================================== */

  /** Method readWellData()
   * <p> This method will read the Well Information from the KGS DB xml stream
   * @return st  = The Well Header Data List Structure
   */

  public static iqstratHeadersListStruct readWellData()
  {
	iqstratHeadersListStruct st = null;

    ReadHeadersXMLFile pRead = new ReadHeadersXMLFile(
		                            iqstrat.io.ReadHeadersXMLFile.URL);

    st = pRead.Process( grid.gridStruct.BRINE_WELLS );

    String sError = pRead.GetError();
    if (sError.length() > 0)
      JOptionPane.showMessageDialog( (Component)null,
                                     sError,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE );
    return (st);
  }

  /* ===================================================================== *
   * -------------------------- LOAD BRINE DATA -------------------------- *
   * ===================================================================== */

  /** Method readFluidLevelData()
   * <p> This method will read the Fluid Level Data from the KGS DB xml stream
   * @return st    = The Brine Data List Structure
   */

  public static fluidListStruct readFluidLevelData()
  {
	fluidListStruct st = null;

  	ReadFluidLevelDataXMLFile pRead =
	  new ReadFluidLevelDataXMLFile(fluid.io.ReadFluidLevelDataXMLFile.URL);

    st = pRead.Process( grid.gridStruct.FLUID_DATA );

    String sError = pRead.GetError();
    if (sError.length() > 0)
      JOptionPane.showMessageDialog( (Component)null,
                                     sError,
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE );
    return (st);
  }
}

/*
 *  @version 1.1 03/21/2016
 *  @author  John Victorine
 */
