/*
 * @gridPlotAnimationFrame.java Version 1.1 02/26/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.plot;

import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;

import grid.plot.gridPlotMappingListStruct; // Grid Plot Mapping List Structure
import grid.plot.gridPlotAnimationPanel;

/** Class gridPlotAnimationFrame
 *  <p> This class will display the Cross Plot Frame
 *
 *  @version 1.1 02/26/2016
 *  @author  John R. Victorine
 */

public class gridPlotAnimationFrame extends JFrame //implements ActionListener
{
  private int                       iAction = 0;
  private gridPlotMappingListStruct stPlot  = null;
  private gridPlotAnimationPanel    panel   = null;

  /** Constructor gridPlotAnimationFrame()
   *  <p> This is the constructor for this class
   *  @param stPlot = Grid Plot Mapping List Structure
   */

  public gridPlotAnimationFrame( gridPlotMappingListStruct stPlot )
  {
    try
    {
	  this.stPlot = stPlot;

      jbInit();

      addWindowListener( new gridPlotAnimationFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    JScrollPane jScrollPane = new JScrollPane();

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle("Gridding & Mapping Plot Animation");

    if (stPlot != null)
    {
      panel = new gridPlotAnimationPanel( stPlot );

      this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
      jScrollPane.getViewport().add(panel, null);
	}

    // Add a Window Listener to close dialog

    this.addWindowListener( new WindowAdapter()
    {
      public void windowClosing(WindowEvent e) { close(); }
    });

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(1300, 750));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation(10,  //(d.width - this.getSize().width)/2,
                     (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD gridPlotAnimationFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class gridPlotAnimationFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      // NO ACTION
    }
  }

  /** Method close()
   *  <p> This method will force deallocation of memory in Java
   */

  public void close()
  {
    stPlot  = null;

    if (panel != null)
      panel.close();
    panel = null;

    dispose();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /* ================================================================== *
   * ------------------------- SET METHODS ---------------------------- *
   * ================================================================== */

  /** Method setWellLabel()
   *  <p> This method will set the type of label to display with the well symbols
   */

  public void setWellLabel(int iLabel)
  {
	if (panel != null) { panel.setWellLabel(iLabel); }
  }

  /* ================================================================== *
   * ------------------- ANIMATION ACTION METHODS --------------------- *
   * ================================================================== */

  /** Method startAnimation() throws Exception
   *  <p> Starts the timer that generates action events
   * @throws Exception - exception starting the timer
   */

  public void startAnimation()
  {
	if (panel != null)
	{
      try
      {
  	    panel.startAnimation();
	  }
	  catch(Exception e) {}
	}
  }

  /** Method pauseAnimation()
   *  <p> Stops the timer that generate action events
   */

  public void pauseAnimation()
  {
	if (panel != null)
	{
      try
      {
  	    panel.pauseAnimation();
	  }
	  catch(Exception e) {}
	}
  }

  /** Method stopAnimation() throws Exception
   *  <p> Stops the timer that generate action events
   *      and resets the current image number to 0
   * @throws Exception - exception stopping the timer
   */

  public void stopAnimation()
  {
	if (panel != null)
	{
      try
      {
  	    panel.stopAnimation();
	  }
	  catch(Exception e) {}
	}
  }

  /** Method stepAction()
   *  <p> This method will issue a step action to the plot
   */

  public void stepAction(int iStep) { panel.stepAction(iStep); }
}

/*
 *  @version 1.1 02/26/2016
 *  @author  John Victorine
 */
