/*
 * @gridPlotMapping.java Version 1.1 07/08/2015
 *
 * Copyright (c) 2002 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.plot;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;

import plot.plotWellStatusStruct;
import plot.BriggsCube;

import grid.plot.gridPlotMappingStruct;
import grid.plot.gridPlotAverage;

/** Class gridPlotMapping
 *  <p> This Plot routine will plot the Computed Volumetrics as a color map.
 *
 *  @version 1.1 07/08/2015
 *  @author  John R. Victorine
 */

public class gridPlotMapping extends Canvas implements MouseMotionListener
{
  public static final int NONE   = 0;  // No Label
  public static final int NUMBER = 1;  // Order Number
  public static final int API    = 2;  // API Number
  public static final int LEASE  = 3;  // Lease Name

  public static final int _RED   = 0;
  public static final int _GREEN = 1;
  public static final int _BLUE  = 2;

  private int iLabel = API; //NONE;  // Indicator to plot Well Label

  private gridPlotMappingStruct stPlot      = null;
  private plotWellStatusStruct  stSymbols   = new plotWellStatusStruct();

  private gridPlotAverage       plotAvg     = null;
  private BriggsCube            plotCube    = null;

  private int                   iXBegin     = 0;
  private int                   iYBegin     = 0;
  private int                   iWidth      = 0;
  private int                   iHeight     = 0;
  private int                   iEWidth     = 550;

  private int                   iTotalWells = 0;
  private int                   iX[]        = null;
  private int                   iY[]        = null;
  private int                   iZ[]        = null;
  private int                   iSymbol[]   = null;
  private int                   iTriangle[] = null;
  private int                   iWellColor  = -1;

  private double                dPI = 3.1415927;
  private double                dXMax = 0.0;
  private double                dXMin = 0.0;
  private double                dYMax = 0.0;
  private double                dYMin = 0.0;

  private double                dMaximum   = 0.0;
  private double                dMinimum   = 0.0;
  private int                   iMaxColors = 768;

  private int                   iDataR[][];
  private int                   iDataG[][];
  private int                   iDataB[][];

  private double                dValue = 0.0;
  private int                   iValue = 0;

  private String                sZoneLabel = new String("");
  private JTextField            txt        = null;

  public static final int   _EXCELLENT   = 0;  // Gold   - Excellent
  public static final int   _GOOD        = 1;  // Silver - Good
  public static final int   _FAIR        = 2;  // Copper - Fair
  public static final int   _POOR        = 3;  // Lead   - Poor
  public static final int   _SELECTED    = 4;  // SELECTED

  public static final int   _HALO[][] = {
	{ 217, 217, 25 },  // Gold
	{ 230, 232, 250 }, // Silver
	{ 217, 135, 25 },  // Fair
	{ 100, 100, 100 }, // Poor
	{ 238, 130, 238 }  // Selected
  };

  public static final String LEVEL[] =
  {
	"Excellent - greater than 95% accepted",
	"Good - greater than 60% accepted",
	"Fair - greater than 15% accepted",
	"Poor - tops selected",
	"Well Selected for Cross Section"
  };

  /** Constructor gridPlotMapping()    INITIALIZE CLASS ONLY
   *  <p> This is the Constructor for this class.
   */

  public gridPlotMapping()
  {
    iWidth  = 300;
    iHeight = 500;
  }

  /** Constructor gridPlotMapping()
   *  <p> This is the Constructor for this class.
   *  @param st     = Mapping Data Structure
   *  @param Width  = The Width of the Plot area
   *  @param Height = The Height of the Plot Area
   */

  public gridPlotMapping( gridPlotMappingStruct st, int Width, int Height )
  {
//    try
//    {
      stPlot     = st;
      sZoneLabel = new String(stPlot.sZone);

      iWidth     = Width-iXBegin;
      iHeight    = Height-iYBegin;

      plotAvg = new gridPlotAverage( st.iRed, st.iGreen, st.iBlue,
                                     st.data, st.sValue,
                                     Width, Height, Width );

      plotCube = new BriggsCube( 400, 0, Width, 250,
		                         st.iRed, st.iGreen, st.iBlue,
		                         st.dMinR, st.dMaxR,
		                         st.dMinG, st.dMaxG,
		                         st.dMinB, st.dMaxB );

//      plotCube = new BriggsCube( 0, 0, iWidth, iHeight,
//		                         st.iRed, st.iGreen, st.iBlue,
//		                         st.dMinR, st.dMaxR,
//		                         st.dMinG, st.dMaxG,
//		                         st.dMinB, st.dMaxB );

    setSize(iWidth+iEWidth, iHeight);
    setBackground(new Color(250, 250, 250));

    // Set the Well Information

    iTotalWells = stPlot.iWells;
    if (iTotalWells > 0)
    {
      setMinMaxPosition();
      setWellData();
      setWellSymbol();
    }

    setZColor();

//      jbInit();
//    }
//    catch (Exception e)
//    {
//      JOptionPane.showMessageDialog( (Component) null,
//                                     "gridPlotMapping(): " + e.getMessage(),
//                                     "ERROR",
//                                     JOptionPane.ERROR_MESSAGE );
//    }
  }

  /** Constructor gridPlotMapping()
   *  <p> This is the Constructor for this class.
   *  @param st     = Mapping Data Structure
   *  @param Width  = The Width of the Plot area
   *  @param Height = The Height of the Plot Area
   *  @param text   = The Text Field that will be updated with the motion of
   *                  the mouse
   */

  public gridPlotMapping( gridPlotMappingStruct st,
                          int Width,
                          int Height,
                          JTextField text )
  {
    try
    {
      stPlot     = st;
      sZoneLabel = new String(stPlot.sZone);

      txt = text;

      iWidth     = Width-iXBegin;
      iHeight    = Height-iYBegin;

      plotAvg = new gridPlotAverage( st.iRed, st.iGreen, st.iBlue,
                                     st.data, st.sValue,
                                     Width, Height, Width );

      plotCube = new BriggsCube( 400, 0, Width, 250,
		                         st.iRed, st.iGreen, st.iBlue,
		                         st.dMinR, st.dMaxR,
		                         st.dMinG, st.dMaxG,
		                         st.dMinB, st.dMaxB );

      jbInit();
    }
    catch (Exception e)
    {
      JOptionPane.showMessageDialog( (Component) null,
                                     "gridPlotMapping(): " + e.getMessage(),
                                     "ERROR",
                                     JOptionPane.ERROR_MESSAGE );
    }
  }

  /** Method jbInit()
   *  <p> This method will create the Linear Plot.
   */

  public void jbInit() throws Exception
  {
    setSize(iWidth+iEWidth, iHeight);
    setBackground(new Color(250, 250, 250));

    // Set the Well Information

    iTotalWells = stPlot.iWells;
    if (iTotalWells > 0)
    {
      setMinMaxPosition();
      setWellData();
      setWellSymbol();
    }

    setZColor();

    addMouseMotionListener(this);
  }

  /** Method close()
   * <p> This method will set the global variables to null in order to force
   *     JAVA to free the memory allocated for this panel.
   */

  public void close()
  {
    stPlot     = null;
    stSymbols  = null;

    iX         = null;
    iY         = null;
    iZ         = null;
    iSymbol    = null;
    iTriangle  = null;

    iDataR     = null;
    iDataG     = null;
    iDataB     = null;

    sZoneLabel = null;
    txt        = null;

    if (plotAvg != null)
      plotAvg.close();
    plotAvg    = null;

    if (plotCube != null)
      plotCube.close();
    plotCube   = null;
  }

  /* ------------------------------------------------------------------------ *
   * -------------------------- Get Data Methods ----------------------------
   * ------------------------------------------------------------------------ */

   /** Method getWidth()
    *  <p> This method will return the actual width of the plot so the frame can
    *      be set to the correct dimensions
    */

  public int getWidth() { return (iWidth + iEWidth); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* ------------------------------------------------------------------------ *
   * -------------------------- Set Data Methods ----------------------------
   * ------------------------------------------------------------------------ */

  /** Method setMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   */

  private void setMinMaxPosition()
  {
    int    iTemp  = 0;
    int    iFound = 0;

    if ((stPlot.dXMin == 0.0) && (stPlot.dXMax == 0.0) &&
        (stPlot.dYMin == 0.0) && (stPlot.dYMax == 0.0))
    {
      for (int i=0; i<iTotalWells; i++)
      {
        if ((Math.abs(stPlot.dX[i]) > 0.0) && (Math.abs(stPlot.dY[i]) > 0.0))
        {
          if (iFound == 0)
          {
            dXMin = stPlot.dX[i];
            dXMax = dXMin;

            dYMin = stPlot.dY[i];
            dYMax = dYMin;

            iFound = 1;
          }

          if (stPlot.dX[i] < dXMin)
            dXMin = stPlot.dX[i];

          if (stPlot.dX[i] > dXMax)
            dXMax = stPlot.dX[i];

          if (stPlot.dY[i] < dYMin)
            dYMin = stPlot.dY[i];

          if (stPlot.dY[i] > dYMax)
            dYMax = stPlot.dY[i];
        }
      }

      double dX = 0.2 * (dXMax - dXMin);
      double dY = 0.2 * (dYMax - dYMin);

      dXMin = dXMin - dX;
      dXMax = dXMax + dX;
      dYMin = dYMin - dY;
      dYMax = dYMax + dY;
    }
    else
    {
      dXMin = stPlot.dXMin;
      dXMax = stPlot.dXMax;
      dYMin = stPlot.dYMin;
      dYMax = stPlot.dYMax;
    }
  }

  /** Method setWellData()
   *  <p> This method will convert the raw data to pixels so the data can be
   *      plotted.
   */

  public void setWellData()
  {
    int iRatio;
    int i;
    int j;

    iX = new int[iTotalWells];
    iY = new int[iTotalWells];

    for (i=0; i<iTotalWells; i++)
    {
      iRatio = (int)((double)iWidth *
                       (stPlot.dX[i]-dXMin) / (dXMax - dXMin));
      iX[i] = iXBegin + iRatio;
    }

    for (i=0; i<iTotalWells; i++)
    {
      iRatio = (int)((double)iHeight *
                       (stPlot.dY[i] - dYMin)/ (dYMax - dYMin));
      iY[i] = iYBegin + iHeight - iRatio;
    }
  }

  /** Method setWellSymbol()
   *  <p> This method will convert the Well Symbol to one of the recognized
   *      symbols.
   */

  private void setWellSymbol()
  {
    iSymbol   = new int[iTotalWells];
    iTriangle = new int[iTotalWells];

    for (int i=0; i<iTotalWells; i++)
    {
      iSymbol[i]   = plot.plotWellStatusStruct.NONE;
      iTriangle[i] = stPlot.iTriangle[i];

      for (int j=0; j<stSymbols.iCount; j++)
      {
        if (stPlot.status[i].equals(stSymbols.status[j][0]))
        iSymbol[i] = stSymbols.iStatus[j];
      }
    }
  }

  /** Method setZColor()
   *  <p> This method will set the color value for the Z Data Value.
   */

  public void setZColor()
  {
    int    i=0;
    int    j=0;
    double dMin = 0.0;
    double dMax = 0.0;
    double dta  = 0.0;

    if ((stPlot.iRows > 0) && (stPlot.iColumns > 0))
    {
      iDataR = new int[stPlot.iRows][stPlot.iColumns];
      iDataG = new int[stPlot.iRows][stPlot.iColumns];
      iDataB = new int[stPlot.iRows][stPlot.iColumns];

      for (i=0; i<stPlot.iRows; i++)
      {
        for (j=0; j<stPlot.iColumns; j++)
        {
		  iDataR[i][j] = 255;
		  iDataG[i][j] = 255;
		  iDataB[i][j] = 255;
//		  iDataR[i][j] = 0;
//		  iDataG[i][j] = 0;
//		  iDataB[i][j] = 0;
        }
      }

      // Red Curve

      if (stPlot.iRed > -1)
      {
        dMin = stPlot.dMinR;
        dMax = stPlot.dMaxR;

        for (i=0; i<stPlot.iRows; i++)
        {
          for (j=0; j<stPlot.iColumns; j++)
          {
			dta = stPlot.dataR[i][j];
			if (dta > dMax) { dta = dMax; }
			if (dta < dMin) { dta = dMin; }

		    iDataR[i][j] = 255 - (int) (255 * (dta - dMin) / (dMax - dMin));
//		    iDataR[i][j] = (int) (255 * (dta - dMin) / (dMax - dMin));
		  }
        }
      }

      // Green Curve

      if (stPlot.iGreen > -1)
      {
        dMin = stPlot.dMinG;
        dMax = stPlot.dMaxG;

        for (i=0; i<stPlot.iRows; i++)
        {
          for (j=0; j<stPlot.iColumns; j++)
          {
			dta = stPlot.dataG[i][j];
			if (dta > dMax) { dta = dMax; }
			if (dta < dMin) { dta = dMin; }

		    iDataG[i][j] = 255 - (int) (255 * (dta - dMin) / (dMax - dMin));
//		    iDataG[i][j] = (int) (255 * (dta - dMin) / (dMax - dMin));
		  }
        }
      }

      // Blue Curve

      if (stPlot.iBlue > -1)
      {
        dMin = stPlot.dMinB;
        dMax = stPlot.dMaxB;

        for (i=0; i<stPlot.iRows; i++)
        {
          for (j=0; j<stPlot.iColumns; j++)
          {
			dta = stPlot.dataB[i][j];
			if (dta > dMax) { dta = dMax; }
			if (dta < dMin) { dta = dMin; }

		    iDataB[i][j] = 255 - (int) (255 * (dta - dMin) / (dMax - dMin));
//		    iDataB[i][j] = (int) (255 * (dta - dMin) / (dMax - dMin));
		  }
        }
      }
    }
  }

  /** Method setLimits()
   *  <p> This method will change the plot limits
   */

  public void setLimits( int iColor, double dMin, double dMax )
  {
	switch (iColor)
	{
	  case _RED:
	    stPlot.dMinR = dMin;
	    stPlot.dMaxR = dMax;
	    break;

	  case _GREEN:
	    stPlot.dMinG = dMin;
	    stPlot.dMaxG = dMax;
	    break;

	  case _BLUE:
	    stPlot.dMinB = dMin;
	    stPlot.dMaxB = dMax;
	    break;
	}

    setZColor();
    repaint();
  }

  /** Method setWellLabel()
   *  <p> This method will set the type of label to display with the well symbols
   */

  public void setWellLabel(int iLabel)
  {
	this.iLabel = iLabel;
	repaint();
  }

  /* ------------------------------------------------------------------------ *
   *  MouseListener Methods required when implementing the MouseListener Class.
   * ------------------------------------------------------------------------ */

  /** Method mouseMoved()
   *  <p> jThis method will retrieve the location of the mouse pointer and
   *       convert the postion to latitude and longitude position. It will then
   *       display the text in the Volumetric Panel.
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event)
  {
    int    iXm = event.getX();
    int    iYm = event.getY();
    int    iXo = 0;
    int    iYo = 0;
    int    iXWidth  = iWidth/(stPlot.iRows-1);
    int    iYHeight = iHeight/(stPlot.iColumns-1);
    int    iFound = 0;
    String sPosition = new String("");
    double delta = (dMaximum -  dMinimum) / (double) iMaxColors;

    if ((iXm > iXBegin) && (iXm < iXBegin + iWidth) &&
        (iYm > iYBegin) && (iYm < iYBegin + iHeight))
    {
      iXo = iXm / iXWidth;
      iYo = (iYBegin + iHeight - iYm) / iYHeight;

      iFound = 0;
      for (int i=0; i<iTotalWells; i++)
      {
        if ((iXm > (iX[i]-5)) && (iXm < (iX[i]+5)) &&
            (iYm > (iY[i]-5)) && (iYm < (iY[i]+5)))
        {
          sPosition = new String(stPlot.sAPI[i]   + "  " + stPlot.sLease[i]);

          iFound = 1;
          if (txt != null)
            txt.setForeground(Color.red);
        }
      }

      if (iFound == 0)
      {
        sPosition = new String(stPlot.sValue    + ": "    + dValue);
        if (txt != null)
          txt.setForeground(Color.black);
      }

      if (txt != null)
        txt.setText(sPosition);
    }
  }

  public void mouseDragged(MouseEvent event) { }
  public void mouseEntered(MouseEvent event) { }
  public void mouseExited(MouseEvent event) { }
  public void mousePressed(MouseEvent event) { }
  public void mouseReleased(MouseEvent event) { }
  public void mouseClicked(MouseEvent event){}


  /* ------------------------------------------------------------------------ *
   * ---------------------------- Draw Methods ------------------------------
   * ------------------------------------------------------------------------ */

  /** Method drawSymbolDescription()
   *  <p> This method will draw the symbols used for the mapping of the wells.
   *  @param g - Graphics pointer.
   */

  public void drawSymbolDescription(Graphics g)
  {
	int i   = 0;
    int iXg = 10;
    int iYg = 10;
    int iYh = 0;

    g.setColor(Color.black);
    g.drawString("           WELL SYMBOLS", iXg, iYg);

    for (i=0; i<stSymbols.iTotalSymbols; i++)
    {
      iXg = 10;
      iYg = 25 + i * 20;
      drawWellSymbol(g, i, iXg, iYg);
      g.drawString(" = " + stSymbols.symbols[i], iXg+20, iYg+5);
    }

    iYg = iYg + 30;

    g.setColor(Color.black);
    g.drawString(" Well Top Picks Confidence Level Halo", iXg, iYg);

    iYh = iYg + 20;

    for (i=0; i<5; i++)
    {
      iXg = 10;
      iYg = iYh + i * 25;
      drawWellSymbol(g, 0, iXg, iYg);
//      drawHalo(g, i, iXg, iYg);

      g.setColor(Color.black);
      g.drawString(" = " + LEVEL[i], iXg+20, iYg+5);
	}
  }

  /** Method drawHalo()
   *  <p> This method will draw a halo around well symbol with the color
   *      representing the status of the well data.
   */

  public void drawHalo(Graphics g, int iColor, int iXp, int iYp)
  {
	if (iColor > -1)
	{
      g.setColor(new Color(_HALO[iColor][0], _HALO[iColor][1], _HALO[iColor][2]));
      g.drawOval(iXp-10, iYp-10, 20, 20);
      g.drawOval(iXp-9,  iYp-9,  18, 18);
    }
  }

  /** Method drawWellSymbol()
   *  <p> This method will draw the Well Symbol.
   *  @param g - Graphics pointer.
   *  @param iSymbol - The symbol Identifier to use in plotting the well Symbol
   *  @param iXp     - X Position of Well
   *  @param iYp     - Y Position of Well
   */

  public void drawWellSymbol(Graphics g, int iSymbol, int iXp, int iYp)
  {
//    if (iWellColor == 1)
//      g.setColor(Color.lightGray);
//    else
//      g.setColor(Color.darkGray);

    if (iWellColor == -1)
    {
      g.setColor(Color.white);
      g.fillOval(iXp-6, iYp-6, 12, 12);
    }

    switch (iSymbol)
    {
      case plot.plotWellStatusStruct.NONE:
        if (iWellColor == -1) g.setColor(Color.black);
        g.drawOval(iXp-5, iYp-5, 10, 10);
        break;

      case plot.plotWellStatusStruct.DRY:
        if (iWellColor == -1) g.setColor(Color.black);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.DRY_OIL:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-7);
        g.drawLine(iXp-5, iYp,   iXp-7, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+7);
        g.drawLine(iXp+5, iYp,   iXp+7, iYp);
        break;

      case plot.plotWellStatusStruct.DRY_GAS:
        if (iWellColor == -1) g.setColor(Color.red);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-8);
        g.drawLine(iXp-5, iYp,   iXp-8, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+8);
        g.drawLine(iXp+5, iYp,   iXp+8, iYp);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        break;

      case plot.plotWellStatusStruct.DRY_OIL_GAS:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-8);
        g.drawLine(iXp-5, iYp,   iXp-8, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+8);
        g.drawLine(iXp+5, iYp,   iXp+8, iYp);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        break;

      case plot.plotWellStatusStruct.OIL:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillOval(iXp-5, iYp-5, 10, 10);
        break;

      case plot.plotWellStatusStruct.GAS:
        if (iWellColor == -1) g.setColor(Color.red);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp+3, iYp+3, iXp+7, iYp+7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.OIL_GAS:
        if (iWellColor == -1) g.setColor(Color.green);
        g.fillOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp+3, iYp+3, iXp+7, iYp+7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SUSPEND:
        if (iWellColor == -1) g.setColor(Color.black);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        break;

      case plot.plotWellStatusStruct.SUSPEND_OIL:
        if (iWellColor == -1) g.setColor(Color.green);
        g.drawOval(iXp-5, iYp-5, 10, 10);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SUSPEND_GAS:
        if (iWellColor == -1) g.setColor(Color.red);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SUSPEND_OIL_GAS:
        if (iWellColor == -1) g.setColor(Color.green);
        g.drawOval(iXp-5, iYp-5, 10, 10);
        g.fillArc(iXp-5, iYp-5, 10, 10, 180, 180);

        g.drawLine(iXp+3, iYp-3, iXp+7, iYp-7);
        g.drawLine(iXp-3, iYp+3, iXp-7, iYp+7);
        g.drawLine(iXp-3, iYp-3, iXp-7, iYp-7);

        g.drawLine(iXp,   iYp-5, iXp,    iYp-10);
        g.drawLine(iXp-5, iYp,   iXp-10, iYp);
        g.drawLine(iXp,   iYp+5, iXp,    iYp+10);
        g.drawLine(iXp+5, iYp,   iXp+10, iYp);
        break;

      case plot.plotWellStatusStruct.SWD:
        if (iWellColor == -1) g.setColor(Color.blue);
        g.drawOval(iXp-5, iYp-5, 10, 10);

        g.drawLine(iXp,   iYp-5, iXp,   iYp+5);
        g.drawLine(iXp-5, iYp,   iXp+5, iYp);
        break;

      case plot.plotWellStatusStruct.OUTCROP:
        if (iWellColor == -1) g.setColor(Color.blue);
        g.drawRect(iXp-5, iYp-5, 10,10);
        break;

    }

    g.setColor(Color.black);
  }

  /** Mtehod drawTriangle()
   *  <p> This method will draw a Triangle around a well if it is to show
   *      a specific region is perforated.
   *  @param g - Graphics pointer.
   *  @param iXp     - X Position of Well
   *  @param iYp     - Y Position of Well
   */

  public void drawTriangle(Graphics g, int iTriangle, int iXp, int iYp)
  {
//    if (iWellColor == 1)
//      g.setColor(Color.lightGray);
//    else
      g.setColor(Color.darkGray);

    if (iTriangle == 1)
    {
      g.drawLine(iXp-15, iYp+10, iXp+15, iYp+10);
      g.drawLine(iXp,    iYp-20, iXp+15, iYp+10);
      g.drawLine(iXp,    iYp-20, iXp-15, iYp+10);
    }

    g.setColor(Color.black);
  }

  /** Method drawWells()
   *  <p> This method will draw well Symbols on the panel.
   *  @param g - Graphics pointer.
   */

  public void drawWells(Graphics g)
  {
    int i, j;

    g.setColor(Color.black);

    for (i=0; i<iTotalWells; i++)
    {
      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
//          if (stPlot.dZ[i] > (dMaximum - dMinimum) / 2)
//            iWellColor = 1;
//          else
//            iWellColor = 0;

          drawWellSymbol(g, iSymbol[i], iX[i], iY[i]);
          drawTriangle(g, iTriangle[i], iX[i], iY[i]);
      }
    }
  }

  /** Method drawWellLabel()
   *  <p> This method will draw well Labels on Map.
   *  @param g - Graphics pointer.
   */

  public void drawWellLabel(Graphics g)
  {
    int    i, j;
    String sLabel = new String("");
    String sName  = "";
    String sName2 = "";
    int    iC[]   = { -1,  -1,  -1 };
    double dC[]   = {0.0, 0.0, 0.0};
    int    iV[]   = {255, 255, 255};
    double dMin   = 0.0;
    double dMax   = 0.0;

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    g.setColor(Color.black);

    for (i=0; i<iTotalWells; i++)
    {
//      for (j=0; i<3; i++)
//      {
//		iC[j] = -1;
//		dC[j] = 0.0;
//	  }

      if ((iX[i] >= iXBegin) && (iX[i] <= iXBegin+iWidth) &&
          (iY[i] >= iYBegin) && (iY[i] <= iYBegin+iHeight))
      {
//          if (stPlot.dZ[i] > (dMaximum - dMinimum) / 2)
//            iWellColor = 1;
//          else
//            iWellColor = 0;

          switch (iLabel)
          {
            case NONE:  // None
              sLabel = new String("");
              break;

            case NUMBER:  // Order Number
              sLabel = new String("" + i);
              break;

            case API:  // API Number
              sLabel = new String(stPlot.sAPI[i]);
              break;

            case LEASE:  // Lease Name
              sName  = new String(stPlot.sLease[i].toLowerCase());
              sLabel = new String(sName.replaceAll("wellington unit", "WU"));
//              sLabel = new String(stPlot.sLease[i]);
              break;
          }

//          if (iWellColor == 1)
//            g.setColor(Color.lightGray);
//          else
            g.setColor(Color.darkGray);

//          g.drawString(sLabel, iX[i]+12 , iY[i]);

 		  iC[0] = stPlot.iRed;
		  dC[0] = stPlot.dZR[i];

          iC[1] = -1;
          dC[1] = 0.0;

          iC[2] = -1;
          dC[2] = 0.0;
          if (stPlot.iGreen != stPlot.iRed)
          {
			iC[1] = stPlot.iGreen;
			dC[1] = stPlot.dZG[i];
		  }

		  if ((stPlot.iBlue != stPlot.iRed) && (stPlot.iBlue != stPlot.iGreen))
		  {
			iC[2] = stPlot.iBlue;
			dC[2] = stPlot.dZB[i];
		  }

          if ((iC[1] == -1) && (iC[2] == -1))
            g.setColor(Color.red);
          else
          {
			for (int k=0; k<3; k++)
			{
			  switch (k)
			  {
			    case 0:
                 dMin = stPlot.dMinR;
                 dMax = stPlot.dMaxR;
                 break;
			    case 1:
                 dMin = stPlot.dMinG;
                 dMax = stPlot.dMaxG;
                 break;
			    case 2:
                 dMin = stPlot.dMinB;
                 dMax = stPlot.dMaxB;
                 break;
			  }

		      iV[k] = 255 - (int) (255 * (dC[k] - dMin) / (dMax - dMin));
		      if (iV[k] < 0) { iV[k] = 0; }
		      if (iV[k] > 255) { iV[k] = 255; }
			}

			if (((iV[0] < 100) && (iV[1] < 100)) ||
			    ((iV[1] < 100) && (iV[2] < 100)) ||
			    ((iV[0] < 100) && (iV[2] < 100)))
			{
              g.setColor(Color.lightGray);
		    }
		  }

          g.drawString(sLabel, iX[i]+12 , iY[i]);
		  for (j=0; j<3; j++)
		  {
			if (iC[j] > -1)
			{
              sName2 = new String(fluid.fluidStandardTools.CURVES[iC[j]][1] + ": " + dC[j]);
              if (dC[j] > 0.0)
                g.drawString(sName2, iX[i]+12 , iY[i]+(j+1)*13);
			}
		  }

      }
    }
  }

  /** Method drawGrid()
   *  <p> This method will create a grid.
   *  @param g - Graphics pointer.
   */

  public void drawGrid(Graphics g)
  {
    int    iXo = 0;
    int    iYo = 0;
    int    iXWidth  = iWidth/(stPlot.iRows-1);
    int    iYHeight = iHeight/(stPlot.iColumns-1);

    g.setColor(Color.lightGray);
    for (int i=0; i<stPlot.iRows; i++)
    {
      iXo = iXBegin + i * iXWidth;

      for (int j=0; j<stPlot.iColumns; j++)
      {
        iYo = iYBegin + iHeight - (j+1) * iYHeight;
        g.drawRect(iXo, iYo, iXWidth, iYHeight);
      }
    }
  }

  /** Method drawSelection()
   *  <p> This method will draw a line on the Color Scale of the value of
   *      the selected cell.
   *  @param g - Graphics Handle;
   */

  public void drawSelection(Graphics g)
  {
    int    iScale = iMaxColors / (iHeight-51);
    int    i = iValue * iScale;

    if (i>iMaxColors)
      i = iMaxColors;

    g.setColor(Color.red);
    g.drawLine(iXBegin+iWidth+50,  iYBegin+iHeight-i,
               iXBegin+iWidth+100, iYBegin+iHeight-i);
  }

  /** Method drawCircles()
   *  <p> This method will draw circles around the different levels
   *      of the monitoring wells
   *  @param g - Graphics Handle;
   */

  public void drawCircles(Graphics g)
  {
	double dXo    = 0.0;
	double dYo    = 0.0;
	int    iRatio = 0;
	int    iXo    = 0;
	int    iYo    = 0;
	int    iRo[]  = { 0, 0, 0 };
	int    iR2    = 0;

    double dX1    = 0.0;
    double dY1    = 0.0;
	int    iX1    = 0;
	int    iY1    = 0;

	if (stPlot != null)
	{
	  dXo = stPlot.dXo;
	  dYo = stPlot.dYo;

      iRatio = (int)((double)iWidth *  (dXo-dXMin) / (dXMax - dXMin));
      iXo    = iXBegin + iRatio;

      iRatio = (int)((double)iHeight * (dYo - dYMin)/ (dYMax - dYMin));
      iYo    = iYBegin + iHeight - iRatio;

      for (int i=0; i<iTotalWells; i++)
      {
		if (stPlot.sLease[i].toLowerCase().equals("wellington unit 62"))
		{
		  dX1 = stPlot.dX[i];
		  dY1 = stPlot.dY[i];

          iRatio = (int)((double)iWidth *  (dX1-dXMin) / (dXMax - dXMin));
          iX1    = iXBegin + iRatio;

          iRatio = (int)((double)iHeight * (dY1 - dYMin)/ (dYMax - dYMin));
          iY1    = iYBegin + iHeight - iRatio;

          iRo[0]    = (int) Math.sqrt( (double) ((iX1-iXo)*(iX1-iXo) + (iY1-iYo)*(iY1-iYo)));
		}

		if (stPlot.sLease[i].toLowerCase().equals("wellington unit 47"))
		{
		  dX1 = stPlot.dX[i];
		  dY1 = stPlot.dY[i];

          iRatio = (int)((double)iWidth *  (dX1-dXMin) / (dXMax - dXMin));
          iX1    = iXBegin + iRatio;

          iRatio = (int)((double)iHeight * (dY1 - dYMin)/ (dYMax - dYMin));
          iY1    = iYBegin + iHeight - iRatio;

          iRo[1]    = (int) Math.sqrt( (double) ((iX1-iXo)*(iX1-iXo) + (iY1-iYo)*(iY1-iYo)));
		}

		if (stPlot.sLease[i].toLowerCase().equals("wellington unit 44"))
		{
		  dX1 = stPlot.dX[i];
		  dY1 = stPlot.dY[i];

          iRatio = (int)((double)iWidth *  (dX1-dXMin) / (dXMax - dXMin));
          iX1    = iXBegin + iRatio;

          iRatio = (int)((double)iHeight * (dY1 - dYMin)/ (dYMax - dYMin));
          iY1    = iYBegin + iHeight - iRatio;

          iRo[2]    = (int) Math.sqrt( (double) ((iX1-iXo)*(iX1-iXo) + (iY1-iYo)*(iY1-iYo)));
		}
      }

      for (int i=0; i<3; i++)
      {
		iR2 = 2*iRo[i];
        g.setColor(Color.black);
        g.drawOval(iXo-iR2/2, iYo-iR2/2, iR2, iR2);
	  }
	}
  }

  /** Method drawData()
   *  <p> This method will paint the Volumetrics values.
   *  @param g - Graphics Handle;
   */

  public void drawData(Graphics g)
  {
    int    iXo = 0;
    int    iYo = 0;
    int    iXWidth  = iWidth/(stPlot.iRows-1);
    int    iYHeight = iHeight/(stPlot.iColumns-1);
    int    iValue = 0;
    int    iRed   = 0;
    int    iGreen = 0;
    int    iBlue  = 0;

    for (int i=0; i<stPlot.iRows; i++)
    {
      iXo = iXBegin + i * iXWidth;

      for (int j=0; j<stPlot.iColumns; j++)
      {
		iRed   = iDataR[i][j];
		iGreen = iDataG[i][j];
		iBlue  = iDataB[i][j];

        iYo    = iYBegin + iHeight - (j+1) * iYHeight;

        g.setColor(new Color(iRed, iGreen, iBlue));
        g.fillRect(iXo, iYo, iXWidth, iYHeight);
      }
    }
  }

  /** Method drawMap()
   *  <p> This method will paint the volumetrics data.
   *  @param g - Graphics Handle;
   */

  public void drawMap(Graphics g)
  {
    drawData(g);
    drawGrid(g);
    drawWells(g);
    drawWellLabel(g);

    drawCircles(g);

    plotAvg.draw(g);
    if ((stPlot.iRed   != stPlot.iGreen) ||
        (stPlot.iGreen != stPlot.iBlue))
      plotCube.draw(g);
  }

  /** Method paint()
   *  <p> This method will paint the volumetrics data.
   *  @param g - Graphics Handle;
   */

  public void paint(Graphics g)
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth+iEWidth, iHeight);
    drawMap(g);
  }
}

/*
 *  @version 1.1 07/08/2015
 *  @author  John Victorine
 */
