/*
 * @gridPlotMappingFrame.java Version 1.1 07/08/2015
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.plot;

import java.awt.*;
import java.awt.image.*;

import javax.swing.*;

import grid.plot.gridPlotMappingStruct;
import grid.plot.gridPlotMapping;

/** Class gridPlotMappingFrame
 *  <p> This class will display the Mapping Plot Data
 *
 *  @version 1.1 07/08/2015
 *  @author  John R. Victorine
 */

public class gridPlotMappingFrame extends JFrame
{
  private gridPlotMapping pnlPlot = null;

  /** CONSTRUCTOR gridPlotMappingFrame()
   *  <p> This is the Constructor for this class.
   *  @param st     = Mapping Data Structure
   *  @param Width  = The Width of the Plot area
   *  @param Height = The Height of the Plot Area
   *  @param color  = selected color shading
   *  @param sPath  = The Path to the Well Status XML File
   */

  public gridPlotMappingFrame( gridPlotMappingStruct st, int Width, int Height)
  {
    try
    {
      jbInit( st, Width, Height );
    }
    catch(Exception e)
    {
      String sMessage = new String("gridPlotMappingFrame(): " + e.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method close()
   *  <p> This method will force Java to deallocate memory
   */

  public void close()
  {
	if (pnlPlot != null)
	  pnlPlot.close();
	pnlPlot = null;

	dispose();
  }

  /** Method jbInit()
   *  <p> This method will create the Well List Frame.
   *  @param st = Mapping Data Structure
   *  @param Width = The Width of the Plot area
   *  @param Height = The Height of the Plot Area
   *  @param color  = selected color shading
   *  @param sPath  = The Path to the Well Status XML File
   *  @throws Exception
   */

  private void jbInit(gridPlotMappingStruct st, int Width, int Height) throws Exception
  {
    JPanel      pnlHeader = new JPanel();
    JPanel      pnlCenter = new JPanel();
    JScrollPane scroll    = new JScrollPane();
    JTextField  txt       = new JTextField();

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle(" Map for " + st.sValue);

    pnlCenter.setLayout(new BorderLayout());

    txt.setFont(new java.awt.Font("SansSerif", 1, 12));
    txt.setDisabledTextColor(Color.black);
    txt.setEditable(false);
    txt.setColumns(50);
    txt.setHorizontalAlignment(SwingConstants.CENTER);

    pnlPlot = new gridPlotMapping( st, Width, Height, txt );
    int iWidth = pnlPlot.getWidth();

    // Attach the widgets to the frame.

    this.getContentPane().add(pnlHeader, BorderLayout.NORTH);
    pnlHeader.add(txt,                   null);

    this.getContentPane().add(pnlCenter, BorderLayout.CENTER);
    pnlCenter.add(scroll,                BorderLayout.CENTER);
    scroll.getViewport().add(pnlPlot,    null);

    // Set the Size of the Frame and Display it

//    this.setSize(600, 450);
    this.setSize(iWidth+50, Height+100);

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);
//    this.setLocation((d.width - 10), (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ------------------------------------------------------------------------ *
   * -------------------------- Get Data Methods ----------------------------
   * ------------------------------------------------------------------------ */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (pnlPlot != null) image = pnlPlot.getImage();

    return (image);
  }

  /* ------------------------------------------------------------------------ *
   * -------------------------- Set Data Methods ----------------------------
   * ------------------------------------------------------------------------ */

  /** Method setLimits()
   *  <p> This method will change the plot limits
   */

  public void setLimits( int iColor, double dMin, double dMax )
  {
	if (pnlPlot != null) { pnlPlot.setLimits( iColor, dMin, dMax ); }
  }

  public void setWellLabel(int iLabel)
  {
	if (pnlPlot != null) { pnlPlot.setWellLabel( iLabel ); }
  }
}

/*
 *  @version 1.1 07/08/2015
 *  @author  John Victorine
 */
