/*
 * @gridPlotMappingStruct.java Version 1.1 07/08/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package grid.plot;

/** Class gridPlotMappingStruct
 *  <p> This Class holds the data needed to perform the mapping plot
 *
 *  @version 1.1 07/08/2015
 *  @author  John R. Victorine
 */

public class gridPlotMappingStruct
{
  // Labels

  public String sZone       = null; // Formation Name of Depth Intervals
  public String sValue      = null; // Value.

  // Well Data to plot on top of the mapping plot.

  public int    iWells      = 0;    // Total Number of Wells that will be displayed
  public double dX[]        = null; // X - Position of Well
  public double dY[]        = null; // Y - Position of Well
  public double dZR[]       = null; // Z - Data Value of Well - Red
  public double dZG[]       = null; // Z - Data Value of Well - Green
  public double dZB[]       = null; // Z - Data Value of Well - Blue
  public String status[]    = null; // Well Status Symbol;
  public String sKID[]      = null; // Well KID;
  public String sAPI[]      = null; // Well API-Number;
  public String sLease[]    = null; // Well Lease Name;
  public int    iTriangle[] = null; // Zone has been perforated.

  // Limits to compute Color

  public int    iRed        = -1;   // Indicator if Curve is present
  public double dMinR       = 0.0;  // Minimum Z Value - Red
  public double dMaxR       = 0.0;  // Maximum Z Value - Red

  public int    iGreen      = -1;   // Indicator if Curve is present
  public double dMinG       = 0.0;  // Minimum Z Value - Green
  public double dMaxG       = 0.0;  // Maximum Z Value - Green

  public int    iBlue       = -1;   // Indicator if Curve is present
  public double dMinB       = 0.0;  // Minimum Z Value - Blue
  public double dMaxB       = 0.0;  // Maximum Z Value - Blue

  // Well Plotting area Dimensions

  public double dXMin       = 0.0;  // Minimum X Position
  public double dXMax       = 0.0;  // Maximum X Position
  public double dXCellSize  = 0.0;  // X Cell Size
  public double dYMin       = 0.0;  // Minimum Y Position
  public double dYMax       = 0.0;  // Maximum Y Position
  public double dYCellSize  = 0.0;  // Y Cell Size

  // Mapping data

  public int    iRows       = 0;    // Total number of Rows
  public int    iColumns    = 0;    // Total number of Columns
  public double dataR[][]   = null; // Array of data cells to be plotted - Red
  public double dataG[][]   = null; // Array of data cells to be plotted - Green
  public double dataB[][]   = null; // Array of data cells to be plotted - Blue

  // Statistics Spread of data

  public double data[][]    = null; // Array of data spread CURVES X Statistics

  // Center Location Wellington KGS 2-32

  public static double Ro         = 6373.0;   // Earth Radius (km)
  public static double r1         = 0.18288;  // Inner Orange Circle 600 ft or 0.18288 km
  public static double r2         = 0.4572;   // Middle Red Circle 1500 ft or 0.4572 km
  public static double r3         = 0.848868; // Outer Green Circle 2785 ft or 0.848868 km

  public static double dLatitude  = 37.3105345;
  public static double dLongitude = -97.4418;
  public static double dUTMx      = 638084.62;
  public static double dUTMy      = 4130256.18;

  public double dXo = 0.0;
  public double dYo = 0.0;

  /** Method delete()
   * <p> This method will set the string variable to null and initialize all the
   *     other variables.
   */

  public void delete()
  {
    // Labels

    sZone       = null; // Formation Name of Depth Intervals
    sValue      = null; // Value.

    // Well Data to plot on top of the mapping plot.

    iWells      = 0;    // Total Number of Wells that will be displayed
    dX          = null; // X - Position of Well
    dY          = null; // Y - Position of Well
    dZR         = null; // Z - Data Value of Well - Red
    dZG         = null; // Z - Data Value of Well - Green
    dZB         = null; // Z - Data Value of Well - Blue
    status      = null; // Well Status Symbol;
    sAPI        = null; // Well API-Number;
    sKID        = null; // Well KID;
    sLease      = null; // Well Lease Name;
    iTriangle   = null; // Zone has been perforated.

    // Limits to compute Color

    iRed        = -1;   // Indicator if Curve is present
    dMinR       = 0.0;  // Minimum Z Value - Red
    dMaxR       = 0.0;  // Maximum Z Value - Red

    iGreen      = -1;   // Indicator if Curve is present
    dMinG       = 0.0;  // Minimum Z Value - Green
    dMaxG       = 0.0;  // Maximum Z Value - Green

    iBlue       = -1;   // Indicator if Curve is present
    dMinB       = 0.0;  // Minimum Z Value - Blue
    dMaxB       = 0.0;  // Maximum Z Value - Blue

    // Well Plotting area Dimensions

    dXMin       = 0.0;  // Minimum X Position
    dXMax       = 0.0;  // Maximum X Position
    dXCellSize  = 0.0;  // X Cell Size
    dYMin       = 0.0;  // Minimum Y Position
    dYMax       = 0.0;  // Maximum Y Position
    dYCellSize  = 0.0;  // Y Cell Size

    // Mapping data

    iRows       = 0;    // Total number of Rows
    iColumns    = 0;    // Total number of Columns
    dataR       = null; // Array of data cells to be plotted - Red
    dataG       = null; // Array of data cells to be plotted - Green
    dataB       = null; // Array of data cells to be plotted - Blue

    // Statistics Spread of data

    data        = null; // Array of data spread CURVES X Statistics
  }
}

/*
 *  @version 1.1 07/08/2015
 *  @author  John Victorine
 */
